import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class AnemometerDemo extends JFrame implements ActionListener, MouseListener {
  protected JLabel lblFactor;
  protected JTextField tfFactor;
  protected JLabel lblCurrCount;
  protected JLabel lblSpeed;
  protected MessageArea msgArea;

  protected double factor;
  protected int currentCount;
  protected int lastRoundCount;

  public static String acFactor="acFactor";

  public AnemometerDemo() {
    super("AnemometerDemo");
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    Container cpane=getContentPane();
    cpane.addMouseListener(this);

    factor=1.0;
    currentCount=0;
    lastRoundCount=0;

    Dimension dim=Toolkit.getDefaultToolkit().getScreenSize();
    setPreferredSize(dim);

    JPanel factorPanel=new JPanel(new FlowLayout());

    lblFactor=new JLabel("Factor: ");
    lblFactor.setBackground(Color.RED);
    factorPanel.add(lblFactor);

    tfFactor=new JTextField(""+factor,4);
    tfFactor.setActionCommand(acFactor);
    tfFactor.addActionListener(this);
    factorPanel.add(tfFactor);

    factorPanel.add(new JLabel("Count:"));
    lblCurrCount=new JLabel("100000");
    factorPanel.add(lblCurrCount);

    factorPanel.add(new JLabel("Speed: "));
    lblSpeed=new JLabel("100000");
    factorPanel.add(lblSpeed);

    factorPanel.add(new JLabel("m/s"));

    cpane.add(factorPanel,BorderLayout.NORTH);

    msgArea=new MessageArea();
    cpane.add(msgArea,BorderLayout.CENTER);

    pack();

    updateCurrentCount();
    updateSpeed();
  }

  public void updateSpeed() {
    lblSpeed.setText(lastRoundCount*factor+"");
    msgArea.setStat(currentCount, lastRoundCount*factor);
    msgArea.repaint();
  }

  public void updateCurrentCount() {
    lblCurrCount.setText(currentCount+"");
    msgArea.setStat(currentCount, lastRoundCount*factor);
    msgArea.repaint();
  }

  public void showInputError() {
    lblFactor.setOpaque(true);
    lblFactor.repaint();
  }

  public void clearInputError() {
    lblFactor.setOpaque(false);
    lblFactor.repaint();
  }

  //
  // Required by ActionListener
  //
  public void actionPerformed(ActionEvent ev) {
    String cmd = ev.getActionCommand();
    if (cmd==acFactor) {
      // set Factor
      try {
        double val=Double.parseDouble(tfFactor.getText());
        if (val<=0) {
          showInputError();
        } else {
          factor=val;
          clearInputError();
          updateSpeed();
        }
      } catch (NumberFormatException ex) {
        showInputError();
      }
    } else {
      // timer expiry
      lastRoundCount=currentCount;
      currentCount=0;
      updateCurrentCount();
      updateSpeed();
    }
  }

  //
  // Required by MouseListener
  //
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {
    ++currentCount;
    updateCurrentCount();
  }
  public void mouseReleased(MouseEvent e) {}

  //
  // Main entry point of program
  //
  public static void main(String[] args) {
    SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          AnemometerDemo mainWindow=new AnemometerDemo();
          mainWindow.setVisible(true);
          javax.swing.Timer timer = new javax.swing.Timer(2000, mainWindow);
          timer.start();
        }
      });
  }
}
